// This is a script that was created to run in Tabular Editor from the C# Script window.
// The purpose of the script is to create calculated tables to support currency conversion patterns.
// The script also includes a way to have dynamic format strings of measures to show currency symbols properly.
// See : https://www.daxpatterns.com/currency-conversion/
// 
// In Tabular Editor, you can press "CTRL+Z" to undo any changes made by scripting (or manually)

// Get 'From Currency' column
var _CurrencyColumns = Model.AllColumns.Where(c => c.Name.Contains("Currency"));
var _CurrencyColumn = SelectColumn(_CurrencyColumns, null, "Select source/from 'Currency Code' column in exchange rate table:");

// DAX for 'Source Currency' and 'Target Currency' tables
// Contains most common Contoso currencies.
// Includes (dynamic) format strings
string _CurrencyTableDAX = @"
FILTER ( 
ADDCOLUMNS (
    DATATABLE (
        ""CurrencyKey"", INTEGER,
        ""Currency Code"", STRING,
        ""Currency"", STRING,
        ""Currency Symbol"", STRING,
        ""Symbol Position"", STRING,
        {
            {1, ""USD"", ""US Dollar"", ""$"", ""prefix""},
            {2, ""AMD"", ""Armenian Dram"", ""֏"", ""suffix""},
            {3, ""AUD"", ""Australian Dollar"", ""$"", ""prefix""},
            {4, ""THB"", ""Thai Baht"", ""฿"", ""prefix""},
            {5, ""CAD"", ""Canadian Dollar"", ""$"", ""prefix""},
            {6, ""DKK"", ""Danish Krone"", ""kr."", ""suffix""},
            {7, ""EUR"", ""EURO"", ""€"", ""prefix""},
            {8, ""HKD"", ""Hong Kong Dollar"", ""$"", ""prefix""},
            {9, ""INR"", ""Indian Rupee"", ""₹"", ""prefix""},
            {10, ""ROL"", ""Romanian Leu"", ""L"", ""suffix""},
            {11, ""MTL"", ""Maltese Lira"", ""₤"", ""prefix""},
            {12, ""TWD"", ""Taiwan Dollar"", ""NT$"", ""prefix""},
            {13, ""BTN"", ""Bhutan Ngultrum"", ""Nu."", ""prefix""},
            {14, ""PKR"", ""Pakistan Rupee"", ""₨"", ""prefix""},
            {15, ""RUB"", ""Russian Ruble"", ""₽"", ""suffix""},
            {16, ""SGD"", ""Singapore Dollar"", ""$"", ""prefix""},
            {17, ""SEK"", ""Swedish Krona"", ""kr"", ""suffix""},
            {18, ""CHF"", ""Swiss Franc"", ""fr."", ""prefix""},
            {19, ""SIT"", ""Slovenian Tolar"", ""SIT"", ""suffix""},
            {20, ""GBP"", ""British Pound"", ""£"", ""prefix""},
            {21, ""KRW"", ""South Korean Won"", ""₩"", ""prefix""},
            {22, ""JPY"", ""Japanese Yen"", ""¥"", ""prefix""},
            {23, ""CNY"", ""Renminbi Yuan"", ""¥"", ""prefix""},
            {24, ""PLN"", ""Polish Zloty"", ""zł"", ""suffix""},
            {25, ""IRR"", ""Iranian Rial"", ""﷼"", ""suffix""},
            {26, ""SYP"", ""Syrian Pound"", ""£"", ""prefix""},
            {27, ""TMM"", ""Turkmenistan Manat"", ""m"", ""suffix""},
            {28, ""KGS"", ""Som"", ""с"", ""suffix""}
        }
    ),
    ""Format String"", 
        IF (
            [Symbol Position] = ""prefix"", 
            [Currency Symbol] & "" #,##0.00"", 
            ""#,##0.00 "" & [Currency Symbol]
        ),
    ""Delta Format String"", 
        IF (
            [Symbol Position] = ""prefix"", 
            [Currency Symbol] & "" #,##0 ↑;"" & [Currency Symbol] & "" #,##0 ↓;"" & [Currency Symbol] & "" #,##0"", 
            ""#,##0 ↑ "" & [Currency Symbol] & "";#,##0 ↓ "" & [Currency Symbol] & "";#,##0 "" & [Currency Symbol]
        )
    ),
    [Currency Code]
    IN
    DISTINCT ( " + _CurrencyColumn.DaxObjectFullName + @")
)
";

// Create 'Source Currency' table
var _SourceCurrency = Model.AddCalculatedTable(
    "Source Currency",
    _CurrencyTableDAX
);

// Create 'Target Currency' table
var _TargetCurrency = Model.AddCalculatedTable(
    "Target Currency",
    @"ALLNOBLANKROW( 'Source Currency' )"
);

// Relate 'Source Currency' to the fact table
foreach ( var _FactColumn in Selected.Columns )
    {
    _FactColumn.RelateTo("Source Currency", "Currency Code");
    Info($"Created relationship with {_FactColumn.DaxObjectFullName}.");
    }

// Relate 'Source Currency' to the table with the exchange rates
_CurrencyColumn.RelateTo("Source Currency", "Currency Code");


// Relate 'Target Currency' to the table with the exchange rates.
_CurrencyColumn = SelectColumn(_CurrencyColumns, null, "Select target/to 'Currency Code' column in exchange rate table:");
_CurrencyColumn.RelateTo("Target Currency", "Currency Code");


Info("Created relationships.");
